# w.c.s. - web application for online forms
# Copyright (C) 2005-2022  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import collections.abc

from quixote import get_publisher

from wcs.qommon import _, errors, get_cfg, sms
from wcs.qommon.form import ComputedExpressionWidget, TextWidget, WidgetList
from wcs.qommon.template import TemplateError
from wcs.variables import LazyList
from wcs.workflows import WorkflowStatusItem, register_item_class, template_on_formdata


class SendSMSWorkflowStatusItem(WorkflowStatusItem):
    description = _('SMS')
    key = 'sendsms'
    category = 'interaction'
    support_substitution_variables = True

    to = []
    body = None

    # don't use roles (de)serializer for "to" field
    to_export_to_xml = None
    to_init_with_xml = None

    @classmethod
    def is_available(cls, workflow=None):
        sms_cfg = get_cfg('sms', {})
        return bool(sms_cfg.get('sender') and sms_cfg.get('passerelle_url'))

    def get_parameters(self):
        return ('to', 'body', 'condition')

    def add_parameters_widgets(self, form, parameters, prefix='', formdef=None, **kwargs):
        super().add_parameters_widgets(form, parameters, prefix=prefix, formdef=formdef, **kwargs)
        if 'to' in parameters:
            form.add(
                WidgetList,
                '%sto' % prefix,
                title=_('To'),
                element_type=ComputedExpressionWidget,
                value=self.to,
                add_element_label=_('Add Number'),
                element_kwargs={'render_br': False},
            )
        if 'body' in parameters:
            form.add(TextWidget, '%sbody' % prefix, title=_('Body'), value=self.body, cols=80, rows=10)

    def get_computed_strings(self):
        yield from super().get_computed_strings()
        yield self.body
        if self.to:
            yield from self.to

    def perform(self, formdata):
        if not self.is_available():
            return
        if not self.to:
            return
        if not self.body:
            return

        recipients = []
        for dest in self.to:
            with get_publisher().complex_data():
                try:
                    dest = self.compute(dest, allow_complex=True, raises=True)
                except Exception:
                    continue
                else:
                    dest = get_publisher().get_cached_complex_data(dest)

            if not dest:
                continue

            if isinstance(dest, list):
                recipients.extend(dest)
            if not isinstance(dest, str):
                if isinstance(dest, LazyList):
                    dest = list(dest)
                if isinstance(dest, collections.abc.Iterable):
                    recipients.extend(dest)
                continue

            if ',' in dest:
                # if the recipient contains a comma consider it as a serie of
                # numbers
                recipients.extend([x.strip() for x in dest.split(',')])
                continue

            recipients.append(dest)

        recipients = list({x for x in recipients if x})  # deduplicate & ignore empty elements
        if not recipients:
            return

        try:
            sms_body = template_on_formdata(
                formdata,
                self.compute(get_publisher().translate(self.body), render=False),
                autoescape=False,
                raises=True,
            )
        except TemplateError as e:
            get_publisher().record_error(
                _('Error in template, sms could not be generated'), formdata=formdata, exception=e
            )
            return

        sms_cfg = get_cfg('sms', {})
        sender = sms_cfg.get('sender', 'AuQuotidien')[:11]
        try:
            sms.SMS.get_sms_class().send(sender, recipients, sms_body)
        except errors.SMSError as e:
            get_publisher().record_error(_('Could not send SMS'), formdata=formdata, exception=e)

    def i18n_scan(self, base_location):
        location = '%sitems/%s/' % (base_location, self.id)
        yield location, None, self.body


register_item_class(SendSMSWorkflowStatusItem)
